<?php

/**
 * @file
 *   Drush integration for DRD.
 */

use Drupal\user\Entity\User;

/**
 * Implementation of hook_drush_command().
 */
function drd_drush_command() {
  $host_options = [
    'host-id' => [
      'description' => t('Host ID'),
    ],
  ];
  $core_options = [
    'core-id' => [
      'description' => t('Core ID'),
    ],
  ];
  $domain_options = [
    'domain-id' => [
      'description' => t('Domain ID'),
    ],
  ];

  $items = [
    'drd-blocks' => [
      'options' => $domain_options + [
        'module' => ['description' => t('The remote module from which to receive blocks')],
        'delta' => ['description' => t('The identifier of a specific block within a module')],
      ],
    ],
    'drd-cron' => [
      'options' => $domain_options,
    ],
    'drd-dnslookup' => [
      'options' => $host_options,
    ],
    'drd-domains-enableall' => [
      'options' => $core_options,
    ],
    'drd-domains-receive' => [
      'options' => $core_options,
    ],
    'drd-errorlogs' => [
      'options' => $domain_options,
    ],
    'drd-flushcache' => [
      'options' => $domain_options,
    ],
    'drd-info' => [
      'options' => $domain_options,
    ],
    'drd-jobscheduler' => [
      'options' => $domain_options,
    ],
    'drd-l10n-update' => [
      'options' => $domain_options,
    ],
    'drd-library-push' => [
      'arguments' => [
        'source' => ['description' => t('Possible values are official|local to select from where the library should be pushed to domains')]
      ],
      'options' => $domain_options,
    ],
    'drd-library-build' => [],
    'drd-maintenancemode' => [
      'arguments' => [
        'mode' => ['description' => t('Possible values are getStatus|on|off')]
      ],
      'options' => $domain_options,
    ],
    'drd-php' => [
      'arguments' => [
        'php' => ['description' => t('Arbitrary PHP code to be executed remotely')]
      ],
      'options' => $domain_options,
    ],
    'drd-ping' => [
      'options' => $domain_options,
    ],
    'drd-projects-usage' => [
      'options' => $domain_options,
    ],
    'drd-projects-status' => [],
    'drd-update' => [
      'options' => $domain_options,
    ],
    'drd-user-credentials' => [
      'arguments' => [
        'uid' => ['description' => t('The user ID for which own or some of the following credentials should be changed')],
      ],
      'options' => $domain_options + [
        'username' => ['description' => t('The user name to be set')],
        'password' => ['description' => t('The password to be set')],
        'status' => ['description' => t('The status to be set')]
      ],
    ],
  ];

  return $items;
}

/**
 * Callback to read one or more drd entities like host, core or domain.
 *
 * @param string $label
 * @param array $properties
 * @return bool|\Drupal\Core\Entity\EntityInterface[]
 */
function _drd_drush_get_entities($label, $properties) {
  $entity_id = drush_get_option($label . '-id');
  if (isset($entity_id)) {
    $entity = \Drupal::entityTypeManager()
      ->getStorage('drd_' . $label)
      ->load($entity_id);
    $entities = empty($entity) ? [] : [$entity];
  }
  else {
    $entities = \Drupal::entityTypeManager()
      ->getStorage('drd_' . $label)
      ->loadByProperties($properties);
  }
  if (empty($entities)) {
    drush_set_error('No ' . $label . ' found!');
    return FALSE;
  }

  return $entities;
}

/**
 * Callback to read one or more domains.
 *
 * @return bool|\Drupal\Core\Entity\EntityInterface[]
 */
function _drd_drush_get_domains() {
  return _drd_drush_get_entities('domain', [
    'status' => 1,
    'installed' => 1,
  ]);
}

/**
 * Callback to read one or more cores.
 *
 * @return bool|\Drupal\Core\Entity\EntityInterface[]
 */
function _drd_drush_get_cores() {
  return _drd_drush_get_entities('core', [
    'status' => 1,
  ]);
}

/**
 * Callback to read one or more hosts.
 *
 * @return bool|\Drupal\Core\Entity\EntityInterface[]
 */
function _drd_drush_get_hosts() {
  return _drd_drush_get_entities('host', [
    'status' => 1,
  ]);
}

/**
 * @param $action_key
 * @return bool|\Drupal\drd\Plugin\Action\BaseEntityInterface
 */
function _drd_drush_get_action($action_key) {
  $action = \Drupal\drd\Plugin\Action\Base::instance($action_key);
  if (!$action || !($action instanceof \Drupal\drd\Plugin\Action\BaseInterface)) {
    drush_set_error('No valid action!');
    return FALSE;
  }
  \Drupal::currentUser()->setAccount(User::load(1));
  return $action;
}

/**
 * Callback to execute an action on one or many drd entities.
 *
 * @param \Drupal\Core\Entity\EntityInterface[] $entities
 * @param string $action_key
 * @param array $arguments
 * @param array $options
 */
function _drd_drush_action($entities, $action_key, $arguments = [], $options = []) {
  if (!$entities) {
    return;
  }

  $action = _drd_drush_get_action($action_key);
  if (!$action) {
    return;
  }

  foreach ($arguments as $key => $value) {
    $action->setActionArgument($key, $value);
  }

  foreach ($options as $key) {
    $value = drush_get_option($key);
    if (isset($value)) {
      $action->setActionArgument($key, $value);
    }
  }

  drush_print('Executing ' . $action->getPluginDefinition()['label']);
  foreach ($entities as $entity) {
    /** @var \Drupal\drd\Entity\BaseInterface $entity */
    drush_print('- on ' . $entity->getName());
    if ($action->executeAction($entity) !== FALSE) {
      if ($action instanceof \Drupal\drd\Plugin\Action\BaseEntityRemote) {
        $response = $action->getResponse();
        if (is_array($response) && !empty($response['output'])) {
          drush_print('  ' . $response['output']);
        }
      }
      drush_print('  ok!');
    }
    else {
      drush_set_error('  failure!');
    }
  }

  drush_print('Completed!');
}

/**
 * Callback to execute a system action.
 *
 * @param string $action_key
 */
function _drd_drush_action_system($action_key) {
  $action = _drd_drush_get_action($action_key);

  if (empty($action) || !($action instanceof \Drupal\drd\Plugin\Action\BaseGlobalInterface)) {
    drush_set_error('No valid local action!');
    return;
  }

  drush_print('Executing ' . $action->getPluginDefinition()['label']);
  if ($action->executeAction()) {
    drush_print('  ok!');
  }
  else {
    drush_set_error('  failure!');
  }

  drush_print('Completed!');
}

/**
 * Callback to execute drd-blocks Drush command.
 */
function drush_drd_blocks() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_blocks', [], ['module', 'delta']);
}

/**
 * Callback to execute drd-cron Drush command.
 */
function drush_drd_cron() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_cron');
}

/**
 * Callback to execute drd-dnslookup Drush command.
 */
function drush_drd_dnslookup() {
  _drd_drush_action(_drd_drush_get_hosts(), 'drd_action_dnslookup');
}

/**
 * Callback to execute drd-domains-enableall Drush command.
 */
function drush_drd_domains_enableall() {
  _drd_drush_action(_drd_drush_get_cores(), 'drd_action_domains_enableall');
}

/**
 * Callback to execute drd-domains-receive Drush command.
 */
function drush_drd_domains_receive() {
  _drd_drush_action(_drd_drush_get_cores(), 'drd_action_domains_receive');
}

/**
 * Callback to execute drd-errorlogs Drush command.
 */
function drush_drd_errorlogs() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_error_logs');
}

/**
 * Callback to execute drd-flushcache Drush command.
 */
function drush_drd_flushcache() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_flush_cache');
}

/**
 * Callback to execute drd-info Drush command.
 */
function drush_drd_info() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_info');
}

/**
 * Callback to execute drd-jobscheduler Drush command.
 */
function drush_drd_jobscheduler() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_job_scheduler');
}

/**
 * Callback to execute drd-l10n-update Drush command.
 */
function drush_drd_l10n_update() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_update_translations');
}

/**
 * Callback to execute drd-library-push Drush command.
 */
function drush_drd_library_push($source) {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_library', ['source' => $source]);
}

/**
 * Callback to execute drd-library-build Drush command.
 */
function drush_drd_library_build() {
  _drd_drush_action_system('drd_action_library_build');
}

/**
 * Callback to execute drd-maintenancemode Drush command.
 */
function drush_drd_maintenancemode($mode) {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_maintenance_mode', ['mode' => $mode]);
}

/**
 * Callback to execute drd-php Drush command.
 */
function drush_drd_php($php) {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_php', ['php' => $php]);
}

/**
 * Callback to execute drd-ping Drush command.
 */
function drush_drd_ping() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_ping');
}

/**
 * Callback to execute drd-projects-usage Drush command.
 */
function drush_drd_projects_usage() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_projects');
}

/**
 * Callback to execute drd-projects-status Drush command.
 */
function drush_drd_projects_status() {
  _drd_drush_action_system('drd_action_projects_status');
}

/**
 * Callback to execute drd-update Drush command.
 */
function drush_drd_update() {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_update');
}

/**
 * Callback to execute drd-user-credentials Drush command.
 */
function drush_drd_user_credentials($uid) {
  _drd_drush_action(_drd_drush_get_domains(), 'drd_action_user_credentials', ['uid' => $uid], ['username', 'password', 'status']);
}
