<?php

/**
 * @file
 * Contains \Drupal\drd\Agent\Action\V8\Update.
 */

namespace Drupal\drd\Agent\Action\V8;

/**
 * Provides a 'Update' code.
 */
class Update extends Base {
  /**
   * @inheritDoc
   */
  public function execute() {
    require_once DRUPAL_ROOT . '/core/includes/install.inc';
    require_once DRUPAL_ROOT . '/core/includes/update.inc';
    drupal_load_updates();
    update_fix_compatibility();

    // Pending hook_update_N() implementations.
    $pending = update_get_update_list();

    // Pending hook_post_update_X() implementations.
    $post_updates = \Drupal::service('update.post_update_registry')->getPendingUpdateInformation();

    $start = array();
    if (count($pending) || count($post_updates)) {
      foreach (array('update', 'post_update') as $update_type) {
        $updates = $update_type == 'update' ? $pending : $post_updates;
        foreach ($updates as $module => $module_updates) {
          if (isset($module_updates['start'])) {
            if (!empty($module_updates['pending'])) {
              $start += [$module => array()];
              $start[$module] = array_merge($start[$module], $module_updates['pending']);
            }
          }
        }
      }
      $this->batch($start);
    }
    return array();
  }

  private function batch($start) {
    $updates = update_resolve_dependencies($start);
    $dependency_map = array();
    foreach ($updates as $function => $update) {
      $dependency_map[$function] = !empty($update['reverse_paths']) ? array_keys($update['reverse_paths']) : array();
    }

    $operations = array();
    foreach ($updates as $update) {
      if ($update['allowed']) {
        // Set the installed version of each module so updates will start at the
        // correct place. (The updates are already sorted, so we can simply base
        // this on the first one we come across in the above foreach loop.)
        if (isset($start[$update['module']])) {
          drupal_set_installed_schema_version($update['module'], $update['number'] - 1);
          unset($start[$update['module']]);
        }
        // Add this update function to the batch.
        $function = $update['module'] . '_update_' . $update['number'];
        $operations[] = array('update_do_one', array($update['module'], $update['number'], $dependency_map[$function]));
        drupal_set_message('Updating ' . $update['module'] . ': version ' . $update['number']);
      }
    }

    // Apply post update hooks.
    $post_updates = \Drupal::service('update.post_update_registry')->getPendingUpdateFunctions();
    if ($post_updates) {
      $operations[] = ['drupal_flush_all_caches', []];
      foreach ($post_updates as $function) {
        $operations[] = ['update_invoke_post_update', [$function]];
      }
    }

    // Lastly, perform entity definition updates, which will update storage
    // schema if needed. If module update functions need to work with specific
    // entity schema they should call the entity update service for the specific
    // update themselves.
    // @see \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface::applyEntityUpdate()
    // @see \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface::applyFieldUpdate()
    $operations[] = array('\Drupal\drd\Agent\Action\V8\Update::entityUpdates', array());

    $batch = array(
      'operations' => $operations,
      'title' => 'Updating',
      'init_message' => 'Starting updates',
      'error_message' => 'An unrecoverable error has occurred. You can find the error message below. It is advised to copy it to the clipboard for reference.',
      'finished' => '\Drupal\drd\Agent\Action\V8\Update::captureUpdateMessages',
    );
    batch_set($batch);
    \Drupal::service('state')->set('system.maintenance_mode', TRUE);
    batch_process();
    \Drupal::service('state')->set('system.maintenance_mode', FALSE);
  }

  public static function entityUpdates(&$context) {
    try {
      \Drupal::entityDefinitionUpdateManager()->applyUpdates();
    }
    catch (\Exception $e) {
      watchdog_exception('update', $e);
    }
  }

  /**
   * Internal function called by update.php
   */
  public static function captureUpdateMessages($success, $results, $operations) {
    foreach ($results as $module => $updates) {
      if ($module != '#abort') {
        foreach ($updates as $number => $queries) {
          foreach ($queries as $query) {
            // If there is no message for this update, don't show anything.
            if (empty($query['query'])) {
              continue;
            }

            if ($query['success']) {
              drupal_set_message($query['query']);
            }
            else {
              drupal_set_message($query['query'], 'error');
            }
          }
        }
      }
    }
  }

}
